
#include "base.h"
#include "enforcer.h"
#include "mysql/dm/connection.h"
#include "mysql/dm/token.h"
#include "mysql/dm/setting.h"
#include "xml/find.h"
#include "util/castable.h"

#include <boost/lexical_cast.hpp>
#include <string>

using namespace mysqlpp;
using namespace boost;
using namespace dm;
using namespace std;

namespace cgi
{
	void base::execute(xmlpp::Document * doc, xmlpp::Node *req, credentials::ptr cred)
	{
		CGI_ENFORCE((_allowed_user_types & cred->user_type()) != 0, "ERROR_INTERNAL_SERVER")
			("User is not authorized for this request");

    _doc = doc;
		_root = doc->get_root_node();
		_request = req;
		_parms = find_exactly_one_node(_request, "parms");
		_payload = find_exactly_one_node(_request, "response/payload");
		_creds = cred;

		CGI_ENFORCE(_payload->get_children().empty(), "ERROR_INTERNAL_SERVER")
			("Node ")(_payload->get_path())(" is not empty");

		do_exec();
	}

	bool base::authenticate(const credentials &cred) const
	{
		CGI_ENFORCE(!cred.token().empty(), "ERROR_INTERNAL_SERVER")("Empty token");

		// Read token from database.
		Query query = connection_holder::instance().get()->query();
		query << "SELECT MINUTE(CURRENT_TIMESTAMP) - MINUTE(timestamp) AS timediff FROM token WHERE "
			<< "client_id = " << cred.client_id() << " AND "
			<< "user_id = " << cred.user_id() << " AND "
			<< "token = '" << cred.token() << "'";
		StoreQueryResult res = query.store();

		if (res.empty()) return false; // token not found -> authentication failure

		CGI_ENFORCE(res.size() == 1, "ERROR_INTERNAL_SERVER")("Found multiple tokens");
		CGI_ENFORCE(!res.at(0)["timediff"].is_null(), "ERROR_INTERNAL_SERVER")
			("Received NULL-value for timediff.");

		// BUG isn't it a bug to do this now? The timestamp of the token
		// if updated to the current time, meaning that even if authentication
		// fails because of timeout, it will succeed if the users tries again
		// right after.
		query << "UPDATE token SET timestamp = CURRENT_TIMESTAMP WHERE "
			<< "client_id = " << cred.client_id() << " AND "
			<< "user_id = " << cred.user_id() << " AND "
			<< "token = '" << cred.token() << "'";
		query.execute();

		int timeout = dm::get_setting<int>("TOKEN_TIMEOUT");
		string timediff_str(res.at(0)["timediff"]);
		CGI_ENFORCE(util::is_castable<int>(timediff_str), "ERROR_INTERNAL_SERVER");
		int timediff = lexical_cast<int>(timediff_str);
		return timediff <= timeout;
	}
}

