
#ifndef __CGI_BASE_H__
#define __CGI_BASE_H__

#include "credentials.h"
#include <boost/shared_ptr.hpp>
#include <libxml++/document.h>

namespace xmlpp
{
	class Node;
}

namespace cgi 
{
	/**
	 * Base class for writing a XML request handler. Instances of this class are used by the executor
	 * in order to handle the different kinds of requests.
	 */
	class base
	{
		public:
			base(credentials::user_t allowed_user_types = (credentials::user_t) (credentials::super | credentials::admin | credentials::analyst | credentials::student)) : 
				_allowed_user_types(allowed_user_types), _creds(), _doc(0), _root(0), _request(0), _payload(0) { }
			virtual ~base() { }

			/**
			 * Execute the request described by node req on the document root using credentials creds.
			 * This method ensures the user has the rights to use this handler, sets the protected
			 * variables below and calls the specialized method do_exec().
			 */
			void execute(xmlpp::Document * doc, xmlpp::Node *req, credentials::ptr creds);
			virtual bool authenticate(const credentials &cred) const;

		protected:
			virtual void do_exec() = 0;

			credentials::user_t _allowed_user_types;
			credentials::ptr _creds;
			xmlpp::Document * _doc;
			xmlpp::Node *_root;
			xmlpp::Node *_request;
			xmlpp::Node *_parms;
			xmlpp::Node *_payload;
	};
}

#endif

