
#include "credentials.h"
#include "xml/find.h"
#include "xml/gettext.h"
#include "mysql/dm/client.h"
#include "mysql/dm/user.h"
#include "mysql/dm/connection.h"

#include <libxml++/nodes/element.h>
#include <libxml++/nodes/textnode.h>

#include <iostream> // TODO: debugging only!!!

using namespace xmlpp;
using namespace dm;
using namespace mysqlpp;

namespace cgi
{
	credentials::credentials(Element *cred)
	{
		CGI_ENFORCE(cred, "ERROR_INTERNAL_SERVER");

		// Get client's name from XML.
		Element *elt = find_exactly_one_node<Element>(cred, "client"); 
		CGI_ENFORCE(elt->get_child_text(), "ERROR_INTERNAL_SERVER")
			("No text found for node: ")(elt->get_path()); 
		_client_name = elt->get_child_text()->get_content();

		// Read client's id from database.
		Query query = connection_holder::instance().get()->query();
		query << "SELECT * FROM client WHERE name = '" << _client_name << "'";
		StoreQueryResult res = query.store();
		CGI_ENFORCE(res.size() == 1, "ERROR_LOGIN")("Unknown client: ")(_client_name); // Assume it is a login (not beautiful, but no problem)
		dm::client client(res.at(0));
		_client_id = client._client_id;

		// Get user's name from XML.
		elt = find_exactly_one_node<Element>(cred, "user"); 
		CGI_ENFORCE(elt->get_child_text(), "ERROR_INTERNAL_SERVER")
			("No text found for node: ")(elt->get_path()); 
		_user_name = elt->get_child_text()->get_content();

		// Read client's id from database.
		query << "SELECT * FROM user WHERE client_id = " << _client_id << " AND " << "name = '" << _user_name << "'";
		res = query.store();
		CGI_ENFORCE(res.size() == 1, "ERROR_LOGIN")("Unknown user: ")(_user_name); // Assume it is a login (not beautiful, but no problem)
		dm::user user(res.at(0));
		_user_id = user._user_id;
		_password = user._password;
		_full_user_name = user._full_name;

		if (user._type == "super")
			_user_type = super;
		else if (user._type == "admin")
			_user_type = admin;
		else if (user._type == "analyst")
			_user_type = analyst;
		else if (user._type == "student")
			_user_type = student;
		else
			CGI_ENFORCE(false, "ERROR_INTERNAL_SERVER")("Unknown user type: ")(user._type);

		// Get token from XML.
		_token = "";
		elt = find_exactly_one_node<Element>(cred, "token"); 
		if (elt->get_child_text())
			_token = elt->get_child_text()->get_content();

		_lang_id = get_text<unsigned>(cred, "language");
	}

	std::string credentials::to_string(user_t t) 
	{
		switch(t)
		{
			case super: return "super";
			case admin: return "admin";
			case analyst: return "analyst";
			case student: return "student";
			default:
				CGI_ENFORCE(false, "ERROR_INTERNAL_SERVER")("Unknown user type: ")(t);
		}

		return ""; // Never reached.
	}
}

